/*
 * LaeqedC.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package dk.thrysoee.laeqed;

import dk.thrysoee.utils.PNGFilterC;
import dk.thrysoee.utils.mru.MruEventC;
import dk.thrysoee.utils.mru.MruListenerI;
import dk.thrysoee.utils.mru.MruMenuC;
import dk.thrysoee.utils.swingworker.SwingWorker;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import javax.swing.UIManager;

/**
 *
 * @author  Jess Thrysoee
 */
public class LaeqedC extends javax.swing.JFrame
{
    private static final Logger log = Logger.getLogger(LaeqedC.class.getName());
    
    private final PreferencesC preferences;
    private final PreferencesDialogC preferencesDialog;
    private final AboutDialogC aboutDialog;
    private final PNGMetaDataDialogC pngMetaDataDialog;
    private final PNGMetaDataHandlerC pngMetaDataHandler;
    private final NativeC nativeRuntime;
    private final JFileChooser fileChooser;
    private final ErrorDialogC errorDialog;
    
    private MruMenuC mruMenu;
    
    private String currentEquation;
    
    
    /**
     * Creates new form LaeqedC
     */
    public LaeqedC()
    {
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
        
        initComponents();
        
        // add editorPane actions to menu
        latexEditorPane.undoActionTo(undoMenuItem);
        latexEditorPane.redoActionTo(redoMenuItem);
        latexEditorPane.cutActionTo(cutMenuItem);
        latexEditorPane.copyActionTo(copyMenuItem);
        latexEditorPane.pasteActionTo(pasteMenuItem);
        latexEditorPane.addSymbolsMenu();
        latexEditorPane.symbolsMenuTo(editMenu, 6);
        
        errorDialog = new ErrorDialogC(this, true);
        
        preferences = new PreferencesC();
        preferencesDialog = new PreferencesDialogC(this, false, preferences);
        preferencesDialog.setTitle(AboutC.nameAndVersion() + " - Preferences");
        
        pngMetaDataDialog = new PNGMetaDataDialogC(this, true, preferences);
        pngMetaDataDialog.setTitle(AboutC.nameAndVersion() + " - Update Preferences");
        
        pngMetaDataHandler = new PNGMetaDataHandlerC(this, pngPanel, latexEditorPane, pngMetaDataDialog, errorDialog);
        pngPanel.setPNGMetaDataHandler(pngMetaDataHandler);
        
        
        aboutDialog = new AboutDialogC(this, true);
        aboutDialog.setTitle(AboutC.nameAndVersion() + " - About");
        
        fileChooser = new JFileChooser();
        fileChooser.setFileFilter(new PNGFilterC());
        
        addMruMenu(preferences);
        
        try
        {
            nativeRuntime = new NativeC(preferences);
        }
        catch (IOException ex)
        {
            throw new RuntimeException("Failed to initalize native runtime", ex);
        }
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        jPanel1 = new javax.swing.JPanel();
        jSplitPane1 = new javax.swing.JSplitPane();
        jScrollPane3 = new javax.swing.JScrollPane();
        latexEditorPane = new dk.thrysoee.laeqed.latexeditor.LatexEditorPaneC();
        jScrollPane2 = new javax.swing.JScrollPane();
        pngPanel = new dk.thrysoee.laeqed.PNGPanelC();
        jPanel2 = new javax.swing.JPanel();
        jSeparator2 = new javax.swing.JSeparator();
        statusBarLabel = new javax.swing.JLabel();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        openMenuItem = new javax.swing.JMenuItem();
        saveAsMenuItem = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        exitMenuItem = new javax.swing.JMenuItem();
        editMenu = new javax.swing.JMenu();
        undoMenuItem = new javax.swing.JMenuItem();
        redoMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        cutMenuItem = new javax.swing.JMenuItem();
        copyMenuItem = new javax.swing.JMenuItem();
        pasteMenuItem = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        preferencesMenuItem = new javax.swing.JMenuItem();
        runMenu = new javax.swing.JMenu();
        runLatexMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle(AboutC.nameAndVersion());
        jSplitPane1.setBorder(null);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane1.setResizeWeight(0.5);
        latexEditorPane.setFont(new java.awt.Font("Monospaced", 0, 12));
        latexEditorPane.setAutoscrolls(false);
        jScrollPane3.setViewportView(latexEditorPane);

        jSplitPane1.setRightComponent(jScrollPane3);

        org.jdesktop.layout.GroupLayout pngPanelLayout = new org.jdesktop.layout.GroupLayout(pngPanel);
        pngPanel.setLayout(pngPanelLayout);
        pngPanelLayout.setHorizontalGroup(
            pngPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 467, Short.MAX_VALUE)
        );
        pngPanelLayout.setVerticalGroup(
            pngPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 99, Short.MAX_VALUE)
        );
        jScrollPane2.setViewportView(pngPanel);

        jSplitPane1.setLeftComponent(jScrollPane2);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jSplitPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 470, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jSplitPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 270, Short.MAX_VALUE)
                .addContainerGap())
        );
        getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

        jPanel2.setLayout(new java.awt.BorderLayout());

        jPanel2.add(jSeparator2, java.awt.BorderLayout.NORTH);

        statusBarLabel.setText(" ");
        jPanel2.add(statusBarLabel, java.awt.BorderLayout.WEST);

        getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

        fileMenu.setMnemonic('F');
        fileMenu.setText("File");
        openMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        openMenuItem.setMnemonic('L');
        openMenuItem.setText("Open...");
        openMenuItem.setToolTipText("Open equation PNG");
        openMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                openMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(openMenuItem);

        saveAsMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        saveAsMenuItem.setMnemonic('A');
        saveAsMenuItem.setText("Save As...");
        saveAsMenuItem.setToolTipText("Save equation PNG");
        saveAsMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                saveAsMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(saveAsMenuItem);

        fileMenu.add(jSeparator3);

        exitMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, java.awt.Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        exitMenuItem.setText("Exit");
        exitMenuItem.setToolTipText("Exit " + AboutC.name());
        exitMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        editMenu.setMnemonic('E');
        editMenu.setText("Edit");
        undoMenuItem.setText("Item");
        editMenu.add(undoMenuItem);

        redoMenuItem.setText("Item");
        editMenu.add(redoMenuItem);

        editMenu.add(jSeparator1);

        cutMenuItem.setText("Item");
        editMenu.add(cutMenuItem);

        copyMenuItem.setText("Item");
        editMenu.add(copyMenuItem);

        pasteMenuItem.setText("Item");
        editMenu.add(pasteMenuItem);

        editMenu.add(jSeparator4);

        preferencesMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        preferencesMenuItem.setMnemonic('P');
        preferencesMenuItem.setText("Preferences...");
        preferencesMenuItem.setToolTipText("Preferences dialog");
        preferencesMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                preferencesMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(preferencesMenuItem);

        menuBar.add(editMenu);

        runMenu.setMnemonic('R');
        runMenu.setText("Run");
        runMenu.setToolTipText("Run");
        runLatexMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_ENTER, java.awt.Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        runLatexMenuItem.setMnemonic('L');
        runLatexMenuItem.setText("Latex");
        runLatexMenuItem.setToolTipText("Run Latex");
        runLatexMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                runLatexMenuItemActionPerformed(evt);
            }
        });

        runMenu.add(runLatexMenuItem);

        menuBar.add(runMenu);

        helpMenu.setMnemonic('H');
        helpMenu.setText("Help");
        aboutMenuItem.setMnemonic('A');
        aboutMenuItem.setText("About");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                aboutMenuItemActionPerformed(evt);
            }
        });

        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        setJMenuBar(menuBar);

        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    private void runLatexMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_runLatexMenuItemActionPerformed
    {//GEN-HEADEREND:event_runLatexMenuItemActionPerformed
        runLatex();
    }//GEN-LAST:event_runLatexMenuItemActionPerformed
    
    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_aboutMenuItemActionPerformed
    {//GEN-HEADEREND:event_aboutMenuItemActionPerformed
        aboutDialog.setVisible();
    }//GEN-LAST:event_aboutMenuItemActionPerformed
    
    private void openMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_openMenuItemActionPerformed
    {//GEN-HEADEREND:event_openMenuItemActionPerformed
        openFileChooser("Open " + AboutC.name() + " equation PNG file");
    }//GEN-LAST:event_openMenuItemActionPerformed
    
    private void saveAsMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_saveAsMenuItemActionPerformed
    {//GEN-HEADEREND:event_saveAsMenuItemActionPerformed
        saveAs("Save " + AboutC.name() + " equation PNG file");
    }//GEN-LAST:event_saveAsMenuItemActionPerformed
    
    private void preferencesMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_preferencesMenuItemActionPerformed
    {//GEN-HEADEREND:event_preferencesMenuItemActionPerformed
        preferencesDialog.setVisible();
    }//GEN-LAST:event_preferencesMenuItemActionPerformed
    
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_exitMenuItemActionPerformed
    {
        dispose();
    }//GEN-LAST:event_exitMenuItemActionPerformed
    
    
    /**
     * run native programs latex and dvipng
     */
    private void runLatex()
    {
        final String input = latexEditorPane.getText();
        
        if ("".equals(input.trim()))
        {
            JOptionPane.showMessageDialog(this, "Equation not found: " +
                    "An equation must be available for latex to run successfully!",
                    "Error - Equation not found",
                    JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        final String statusBarText = "Running Latex...";
        statusBarLabel.setText(statusBarText);
        
        SwingWorker worker = new SwingWorker<PNGImageC, Object>()
        {
            PNGImageC pngImage;
            Exception exception;
            
            public PNGImageC doInBackground()
            {
                try
                {
                    pngImage = nativeRuntime.exec(input);
                }
                catch (NativeExceptionC ex)
                {
                    // defer errorDialog popup to the event dispatching thread
                    exception = ex;
                }
                catch (IOException ex)
                {
                    exception = ex;
                }
                catch (InterruptedException ex)
                {
                    exception = ex;
                }
                
                return pngImage;
            }
            
            protected void done()
            {
                if (exception != null)
                {
                    statusBarLabel.setText(statusBarText + " failed");
                    
                    if (exception instanceof NativeExceptionC)
                    {
                        NativeExceptionC ex = (NativeExceptionC) exception;
                        errorDialog.setVisible(ex.getMessage(), ex.getDetails());
                    }
                    else
                    {
                        errorDialog.setVisible("NativeC Exception", exception);
                    }
                }
                
                else if (pngImage != null)
                {
                    currentEquation = latexEditorPane.getText();
                    try
                    {
                        pngImage.setLatexPreamble(preferences.getLatexPreamble());
                        pngImage.setLatexPostamble(preferences.getLatexPostamble());
                        pngImage.setDvipngOptions(preferences.getDvipngOptions());
                        pngImage.setLatexEquation(currentEquation);
                        
                        pngPanel.setPNGImage(pngImage);
                        
                        statusBarLabel.setText(statusBarText + " done");
                    }
                    catch (IOException ex)
                    {
                        statusBarLabel.setText(statusBarText + " failed");
                        errorDialog.setVisible("Failed to set image PNG", ex);
                    }
                }
                
                clearStatusBarLabel(3000);
            }
        };
        worker.execute();
    }
    
    
    /**
     * Open an equation PNG from chosen file.
     * @param title FileChooser dialog title
     * @return the selected File or null
     */
    private void openFileChooser(String title)
    {
        fileChooser.setDialogTitle(title);
        int res = fileChooser.showOpenDialog(this);
        if(res == JFileChooser.APPROVE_OPTION)
        {
            File file = fileChooser.getSelectedFile();
            if (openPNG(file))
            {
                addMruFile(file);
            }
        }
    }
    
    
    /**
     * Open PNG file.
     *
     * @param file the PNG file.
     * @return true if file was open successfully, false otherwise.
     */
    private boolean openPNG(File file)
    {
        boolean res = pngMetaDataHandler.updateMetaData(file);
        
        if (!res)
        {
            removeMruFile(file);
        }
        
        return res;
    }
    
    
    /**
     * Save PNG to chosen file.
     *
     * @param title FileChooser dialog title
     * @return the selected File or null
     */
    private File saveAs(String title)
    {
        File file = null;
        PNGImageC pngImage = pngPanel.getPNGImage();
        
        if (pngImage != null)
        {
            fileChooser.setDialogTitle(title);
            int res = fileChooser.showSaveDialog(this);
            if(res == JFileChooser.APPROVE_OPTION)
            {
                file = fileChooser.getSelectedFile();
                
                try
                {
                    pngImage.writeTo(file);
                    addMruFile(file);
                    
                }
                catch (IOException ex)
                {
                    errorDialog.setVisible("Failed to save '"+ file.getPath()+"'", ex);
                }
            }
        }
        else
        {
            JOptionPane.showMessageDialog(this,
                    "Rendered equation not available. " +
                    "A successfull render of the equation must precede save!",
                    "Error - Rendered equation not available",
                    JOptionPane.ERROR_MESSAGE);
        }
        
        return file;
    }
    
    
    /**
     * Add MRU menu
     *
     * @param preferences
     */
    private void addMruMenu(final PreferencesC preferences)
    {
        mruMenu = new MruMenuC(9, 60, preferences.getMruFileList());
        mruMenu.addMruListener(new MruListenerI()
        {
            public void mruActionPerformed(MruEventC evt)
            {
                if (openPNG(evt.getFile()))
                {
                    fileChooser.setSelectedFile(evt.getFile());
                    preferences.setMruFileList(mruMenu.getFileList());
                }
            }
        });
        
        fileMenu.add(mruMenu, 1);
    }
    
    
    /**
     * Add file to the MRU menu
     *
     * @param file
     */
    private void addMruFile(File file)
    {
        mruMenu.add(file);
        preferences.setMruFileList(mruMenu.getFileList());
    }
    
    
    /**
     * Remove file from the MRU menu
     *
     * @param file
     */
    private void removeMruFile(File file)
    {
        mruMenu.remove(file);
        preferences.setMruFileList(mruMenu.getFileList());
    }
    
    
    /**
     * Clear status bar label after a <code>delay</code>.
     *
     * @param delay delay in milliseconds
     */
    private void clearStatusBarLabel(int delay)
    {
        Timer timer = new Timer(delay, new ActionListener()
        {
            public void actionPerformed(ActionEvent evt)
            {
                statusBarLabel.setText(" ");
            }
        }
        );
        timer.setRepeats(false);
        timer.start();
    }
    
    
    /**
     * Program main
     *
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        // Mac OS X menubar and brushMetal
        //System.setProperty("apple.laf.useScreenMenuBar", "true");
        //System.setProperty("apple.awt.brushMetalLook", "true");
        //System.setProperty("apple.awt.brushMetalRounded", "true");
        
        java.awt.EventQueue.invokeLater(new Runnable()
        {
            public void run()
            {
                Thread.UncaughtExceptionHandler handler =  new ErrorDialogC(null, "Uncaught Exception", true);
                Thread.setDefaultUncaughtExceptionHandler(handler);
                
                LaeqedC laeqed = new LaeqedC();
                
                // exit application on dispose()
                laeqed.addWindowListener(new WindowAdapter()
                {
                    public void windowClosed(WindowEvent e)
                    {
                        System.exit(0);
                    }
                });
                
                laeqed.setLocationRelativeTo(null);
                laeqed.setVisible(true);
            }
        });
    }
    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JMenuItem copyMenuItem;
    private javax.swing.JMenuItem cutMenuItem;
    private javax.swing.JMenu editMenu;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JSplitPane jSplitPane1;
    private dk.thrysoee.laeqed.latexeditor.LatexEditorPaneC latexEditorPane;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem openMenuItem;
    private javax.swing.JMenuItem pasteMenuItem;
    private dk.thrysoee.laeqed.PNGPanelC pngPanel;
    private javax.swing.JMenuItem preferencesMenuItem;
    private javax.swing.JMenuItem redoMenuItem;
    private javax.swing.JMenuItem runLatexMenuItem;
    private javax.swing.JMenu runMenu;
    private javax.swing.JMenuItem saveAsMenuItem;
    private javax.swing.JLabel statusBarLabel;
    private javax.swing.JMenuItem undoMenuItem;
    // End of variables declaration//GEN-END:variables
    
}

