/*
 * PNGMetaDataHandlerC.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package dk.thrysoee.laeqed;

import dk.thrysoee.laeqed.latexeditor.LatexEditorPaneC;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.imageio.IIOException;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

/**
 * Handle update meta data on PNGImageC and preferences/GUI from file open menu or (dnd) drop.
 *
 * @author Jess Thrysoee
 */
public class PNGMetaDataHandlerC
{
    private final JFrame parent;
    private final PNGMetaDataDialogC pngMetaDataDialog;
    private final PNGPanelC pngPanel;
    private final LatexEditorPaneC latexEditorPane;
    private final ErrorDialogC errorDialog;
    
    
    /** Creates a new instance of PNGMetaDataHandlerC */
    public PNGMetaDataHandlerC(JFrame parent, PNGPanelC pngPanel, LatexEditorPaneC latexEditorPane, PNGMetaDataDialogC pngMetaDataDialog, ErrorDialogC errorDialog)
    {
        this.parent = parent;
        this.pngPanel = pngPanel;
        this.latexEditorPane = latexEditorPane;
        this.pngMetaDataDialog = pngMetaDataDialog;
        this.errorDialog = errorDialog;
    }
    
    public boolean updateMetaData(File file)
    {
        boolean res = false;
        
        try
        {
            PNGImageC pngImage = new PNGImageC(file);
            res = updateMetaData(pngImage);
        }
        catch (FileNotFoundException ex)
        {
            JOptionPane.showMessageDialog(parent,  "File not found: '" + file.getPath() +"'",
                    "Error - File not found",
                    JOptionPane.ERROR_MESSAGE);
        }
        catch (IOException ioex)
        {
            errorDialog.setVisible("PNGMetaDataHandlerC IO", ioex);
        }
        
        return res;
    }
    
    
    public boolean updateMetaData(PNGImageC pngImage)
    {
        
        boolean res = false;
        
        try
        {
            pngMetaDataDialog.setLatexPreamble(pngImage.getLatexPreamble());
            pngMetaDataDialog.setLatexPostamble(pngImage.getLatexPostamble());
            pngMetaDataDialog.setDvipngOptions(pngImage.getDvipngOptions());
            
            if (!pngMetaDataDialog.isNoneSelected())
            {
                pngMetaDataDialog.setVisible();
            }
            
            pngPanel.setPNGImage(pngImage);
            latexEditorPane.setText(pngImage.getLatexEquation());
            res = true;
            
        }
        catch (IIOException iioex)
        {
            JOptionPane.showMessageDialog(parent, "Unrecognized image format: " +
                    "This file is probably not a " + AboutC.name() + " generated equation PNG",
                    "Error - Unrecognized image format",
                    JOptionPane.ERROR_MESSAGE);
        }
        catch (IOException ioex)
        {
            errorDialog.setVisible("PNGMetaDataHandlerC IO", ioex);
        }
        
        return res;
    }
}
