/*
 * PreferencesDialogC.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package dk.thrysoee.laeqed;

import dk.thrysoee.utils.HideOnEscapeC;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 *
 * @author  Jess Thrysoee
 */
public class PreferencesDialogC extends javax.swing.JDialog
{
    private final PreferencesC preferences;
    private final JFileChooser fileChooser;
    
    
    /** Creates new form PreferencesDialogC */
    public PreferencesDialogC(java.awt.Frame parent, boolean modal, PreferencesC preferences)
    {
        super(parent, modal);
        this.preferences = preferences;
        
        initComponents();
        
        initPreferencesListener();
        
        inputLatexPreamble.setCaretPosition(0);
        inputLatexPostamble.setCaretPosition(0);
        
        fileChooser = new JFileChooser();
        
        // cancel as default
        getRootPane().setDefaultButton(cancelButton);
        
        HideOnEscapeC.install(this);
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        defaultsButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        applyButton = new javax.swing.JButton();
        okButton = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jLabel7 = new javax.swing.JLabel();
        inputLatexApp = new javax.swing.JTextField();
        browseLatexButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        inputLatexPreamble = new dk.thrysoee.laeqed.latexeditor.LatexEditorPaneC();
        jScrollPane1 = new javax.swing.JScrollPane();
        inputLatexPostamble = new dk.thrysoee.laeqed.latexeditor.LatexEditorPaneC();
        jPanel2 = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        inputDvipngApp = new javax.swing.JTextField();
        browseDvipngButton = new javax.swing.JButton();
        inputDvipngOptions = new javax.swing.JTextField();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        defaultsButton.setMnemonic('D');
        defaultsButton.setText("Defaults");
        defaultsButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                defaultsButtonActionPerformed(evt);
            }
        });

        cancelButton.setMnemonic('C');
        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cancelButtonActionPerformed(evt);
            }
        });

        applyButton.setMnemonic('A');
        applyButton.setText("Apply");
        applyButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                applyButtonActionPerformed(evt);
            }
        });

        okButton.setMnemonic('O');
        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                okButtonActionPerformed(evt);
            }
        });

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("latex"));
        jLabel7.setText("Application:");

        inputLatexApp.setText(preferences.getLatexApplication());

        browseLatexButton.setText("Browse...");
        browseLatexButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                browseLatexButtonActionPerformed(evt);
            }
        });

        jLabel1.setText("Preamble:");

        jLabel2.setText("Postamble:");

        inputLatexPreamble.setFont(new java.awt.Font("Monospaced", 0, 12));
        inputLatexPreamble.setText(preferences.getLatexPreamble());
        jScrollPane2.setViewportView(inputLatexPreamble);

        inputLatexPostamble.setFont(new java.awt.Font("Monospaced", 0, 12));
        inputLatexPostamble.setText(preferences.getLatexPostamble());
        jScrollPane1.setViewportView(inputLatexPostamble);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(12, 12, 12)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jLabel1)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jLabel7)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jLabel2))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(inputLatexApp, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 277, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(browseLatexButton)
                        .add(14, 14, 14))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 374, Short.MAX_VALUE)
                            .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 374, Short.MAX_VALUE))
                        .addContainerGap())))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel7)
                    .add(inputLatexApp, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(browseLatexButton))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel1)
                    .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 283, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel2)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 90, Short.MAX_VALUE))
                .addContainerGap())
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("dvipng"));
        jLabel5.setText("Application:");

        jLabel6.setText("Options:");

        inputDvipngApp.setText(preferences.getDvipngApplication());

        browseDvipngButton.setText("Browse...");
        browseDvipngButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                browseDvipngButtonActionPerformed(evt);
            }
        });

        inputDvipngOptions.setText(preferences.getDvipngOptions());

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jLabel5)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jLabel6))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                        .add(inputDvipngApp, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 279, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(browseDvipngButton))
                    .add(inputDvipngOptions, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 374, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel5)
                    .add(browseDvipngButton)
                    .add(inputDvipngApp, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel6)
                    .add(inputDvipngOptions, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(14, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(defaultsButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 170, Short.MAX_VALUE)
                        .add(okButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(applyButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(cancelButton))
                    .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );

        layout.linkSize(new java.awt.Component[] {applyButton, cancelButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(31, 31, 31)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(defaultsButton)
                    .add(cancelButton)
                    .add(applyButton)
                    .add(okButton))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    private void browseLatexButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_browseLatexButtonActionPerformed
    {//GEN-HEADEREND:event_browseLatexButtonActionPerformed
        File file = chooseApplication("Select Latex Application");
        if (file != null)
        {
            inputLatexApp.setText(file.getPath());
        }
    }//GEN-LAST:event_browseLatexButtonActionPerformed
    
    private void defaultsButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_defaultsButtonActionPerformed
    {//GEN-HEADEREND:event_defaultsButtonActionPerformed
        //inputLatexApp.setText(preferences.getLatexApplicationDefault());
        
        inputLatexPreamble.setText(preferences.getLatexPreambleDefault());
        inputLatexPreamble.setCaretPosition(0);
        
        inputLatexPostamble.setText(preferences.getLatexPostambleDefault());
        inputLatexPostamble.setCaretPosition(0);
        
        //inputDvipngApp.setText(preferences.getDvipngApplicationDefault());
        
        if (!preferences.getDvipngOptionsDefault().equals(inputDvipngOptions.getText()))
        {
            inputDvipngOptions.setText(preferences.getDvipngOptionsDefault());
        }
        
    }//GEN-LAST:event_defaultsButtonActionPerformed
    
    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
    {//GEN-HEADEREND:event_cancelButtonActionPerformed
        resetAmblesAndOptions();
        inputLatexApp.setText(preferences.getLatexApplication());
        inputDvipngApp.setText(preferences.getDvipngApplication());
        
        setVisible(false);
    }//GEN-LAST:event_cancelButtonActionPerformed
    
    private void applyButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_applyButtonActionPerformed
    {//GEN-HEADEREND:event_applyButtonActionPerformed
        setPreferences();
    }//GEN-LAST:event_applyButtonActionPerformed
    
    private void okButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_okButtonActionPerformed
    {//GEN-HEADEREND:event_okButtonActionPerformed
        setPreferences();
        setVisible(false);
    }//GEN-LAST:event_okButtonActionPerformed
    
    private void browseDvipngButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_browseDvipngButtonActionPerformed
    {//GEN-HEADEREND:event_browseDvipngButtonActionPerformed
        File file = chooseApplication("Select Dvipng Application");
        if (file != null)
        {
            inputDvipngApp.setText(file.getPath());
        }
    }//GEN-LAST:event_browseDvipngButtonActionPerformed
    
    
    /**
     * Persist preferences to the PreferencesC object.
     */
    private void setPreferences()
    {
        preferences.setLatexApplication(inputLatexApp.getText());
        preferences.setLatexPreamble(inputLatexPreamble.getText(), false);
        preferences.setLatexPostamble(inputLatexPostamble.getText(), false);
        
        preferences.setDvipngApplication(inputDvipngApp.getText());
        preferences.setDvipngOptions(inputDvipngOptions.getText(), false);
    }
    
    
    public void resetAmblesAndOptions()
    {
        inputLatexPreamble.setText(preferences.getLatexPreamble());
        inputLatexPreamble.setCaretPosition(0);
        
        inputLatexPostamble.setText(preferences.getLatexPostamble());
        inputLatexPostamble.setCaretPosition(0);
        
        if (!preferences.getDvipngOptions().equals(inputDvipngOptions.getText()))
        {
            inputDvipngOptions.setText(preferences.getDvipngOptions());
        }
    }
    
    
    /**
     * choose application path.
     * @param title FileChooser dialog title
     * @return the selected File or null
     */
    private File chooseApplication(String title)
    {
        File file = null;
        
        fileChooser.setDialogTitle(title);
        int res = fileChooser.showOpenDialog(this);
        if(res == JFileChooser.APPROVE_OPTION)
        {
            file = fileChooser.getSelectedFile();
            if (!file.exists())
            {
                JOptionPane.showMessageDialog(this,
                        "'"+file.getPath()+"' does not exist!",
                        "Error - file does not exist",
                        JOptionPane.ERROR_MESSAGE);
                file = null;
            }
        }
        return file;
    }
    
    
    /**
     * Listen for changes to preferences and update components.
     */
    private void initPreferencesListener()
    {
        preferences.addPropertyChangeListener(new java.beans.PropertyChangeListener()
        {
            public void propertyChange(java.beans.PropertyChangeEvent evt)
            {
                if ("LatexPreamble".equals(evt.getPropertyName()))
                {
                    inputLatexPreamble.setText((String)evt.getNewValue());
                    inputLatexPreamble.setCaretPosition(0);
                }
                if ("LatexPostamble".equals(evt.getPropertyName()))
                {
                    inputLatexPostamble.setText((String)evt.getNewValue());
                    inputLatexPostamble.setCaretPosition(0);
                }
                if ("DvipngOptions".equals(evt.getPropertyName()))
                {
                    inputDvipngOptions.setText((String)evt.getNewValue());
                }
            }
        });
    }
    
    
    /**
     * show dialog
     */
    public void setVisible()
    {
        if (!isVisible())
        {
            pack();
            setLocationRelativeTo(getParent());
            setVisible(true);
        }
        else
        {
            requestFocus();
        }
    }
    
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        java.awt.EventQueue.invokeLater(new Runnable()
        {
            public void run()
            {
                new PreferencesDialogC(new javax.swing.JFrame(), true, new PreferencesC()).setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton applyButton;
    private javax.swing.JButton browseDvipngButton;
    private javax.swing.JButton browseLatexButton;
    private javax.swing.JButton cancelButton;
    private javax.swing.JButton defaultsButton;
    private javax.swing.JTextField inputDvipngApp;
    private javax.swing.JTextField inputDvipngOptions;
    private javax.swing.JTextField inputLatexApp;
    private dk.thrysoee.laeqed.latexeditor.LatexEditorPaneC inputLatexPostamble;
    private dk.thrysoee.laeqed.latexeditor.LatexEditorPaneC inputLatexPreamble;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JButton okButton;
    // End of variables declaration//GEN-END:variables
    
}
