/*
 * LatexEditorPaneC.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package dk.thrysoee.laeqed.latexeditor;

import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsAccentMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsArrowMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsBinaryOperatorMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsDelimiterMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsHebrewMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsMiscellaneousMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsNegatedArrowMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsNegatedRelationOneMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsNegatedRelationTwoMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsRelationOneMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsRelationTwoMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.AmsSpecialMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexAccentMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexArrowMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexBinaryOperatorMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexConstructMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexDelimiterMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexGreekLowercaseMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexGreekUppercaseMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexLogLikeMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexMiscellaneousMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexPunctuationMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexRelationMenuC;
import dk.thrysoee.laeqed.latexeditor.symbolsmenu.LatexVariableSizeMenuC;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.JEditorPane;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;


/**
 *
 * @author Jess Thrysoee
 */
public class LatexEditorPaneC extends JEditorPane
{
    private static final Logger log = Logger.getLogger(LatexEditorPaneC.class.getName());
    
    // Listener for the edits on the current document.
    private final UndoableEditListener undoHandler = new UndoHandler();
    
    // UndoManager that we add edits to.
    private final UndoManager undo = new UndoManager();
    
    private UndoAction undoAction;
    private RedoAction redoAction;
    
    private JPopupMenu popup;
    
    
    /**
     * Creates a new instance of LatexEditorPaneC
     */
    public LatexEditorPaneC()
    {
        setContentType("text/latex");
        setEditorKit(new LatexEditorKitC());
        
        // Add this as a listener for undoable edits.
        getDocument().addUndoableEditListener(undoHandler);
        
        createActions();
        createPopupMenu();
        MouseListener popupListener = new PopupListener();
        addMouseListener(popupListener);
    }
    
    /**
     * Create the pop-up menu
     */
    private void createPopupMenu()
    {
        popup = new JPopupMenu();
        
        // Undo
        popup.add(undoActionTo(new JMenuItem()));
        // Redo
        popup.add(redoActionTo(new JMenuItem()));
        
        popup.addSeparator();
        
        // Cut
        popup.add(cutActionTo(new JMenuItem()));
        // Copy
        popup.add(copyActionTo(new JMenuItem()));
        // Paste
        popup.add(pasteActionTo(new JMenuItem()));
    }
    
    
    public void addSymbolsMenu()
    {
        symbolsMenuTo(popup);
    }
    
    
    /**
     * Add Math symbols menus to the pop-up menu
     */
    public void symbolsMenuTo(JPopupMenu menu)
    {
        menu.addSeparator();
        menu.add(getSymbolMenu());
    }
    
    
    public void symbolsMenuTo(JMenu menu, int menuIndex)
    {
        menu.insertSeparator(menuIndex);
        menu.insert(getSymbolMenu(), ++menuIndex);
    }
    
    
    private JMenu getSymbolMenu()
    {
        
        JMenu symbolMenu = new JMenu("Insert Symbol");
        
        
        symbolMenu.add(new LatexGreekUppercaseMenuC(this));
        symbolMenu.add(new LatexGreekLowercaseMenuC(this));
        
        symbolMenu.add(new LatexBinaryOperatorMenuC(this));
        symbolMenu.add(new LatexRelationMenuC(this));
        symbolMenu.add(new LatexArrowMenuC(this));
        symbolMenu.add(new LatexPunctuationMenuC(this));
        symbolMenu.add(new LatexMiscellaneousMenuC(this));
        symbolMenu.add(new LatexVariableSizeMenuC(this));
        symbolMenu.add(new LatexLogLikeMenuC(this));
        symbolMenu.add(new LatexDelimiterMenuC(this));
        symbolMenu.add(new LatexConstructMenuC(this));
        symbolMenu.add(new LatexAccentMenuC(this));
        
        JMenu amsMenu = new JMenu("AMS");
        symbolMenu.add(amsMenu);
        
        amsMenu.add(new AmsHebrewMenuC(this));
        amsMenu.add(new AmsArrowMenuC(this));
        amsMenu.add(new AmsNegatedArrowMenuC(this));
        
        amsMenu.add(new AmsRelationOneMenuC(this));
        amsMenu.add(new AmsRelationTwoMenuC(this));
        amsMenu.add(new AmsNegatedRelationOneMenuC(this));
        amsMenu.add(new AmsNegatedRelationTwoMenuC(this));
        
        amsMenu.add(new AmsBinaryOperatorMenuC(this));
        amsMenu.add(new AmsMiscellaneousMenuC(this));
        amsMenu.add(new AmsAccentMenuC(this));
        amsMenu.add(new AmsDelimiterMenuC(this));
        amsMenu.add(new AmsSpecialMenuC(this));
        
        return symbolMenu;
    }
    
    
    /**
     * PopupListener
     */
    class PopupListener extends MouseAdapter
    {
        public void mousePressed(MouseEvent e)
        {
            showPopup(e);
        }
        
        public void mouseReleased(MouseEvent e)
        {
            showPopup(e);
        }
        
        private void showPopup(MouseEvent e)
        {
            if (e.isPopupTrigger())
            {
                popup.show(e.getComponent(), e.getX(), e.getY());
            }
        }
    }
    
    
    /**
     * only insert text if it is different from the old text. This is
     * done to prevent superflous undo events. Not a viable solution if the
     * texts get large.
     */
    @Override
    public void setText(String t)
    {
        String oldText = getText();
        if (oldText != null && !oldText.equals(t))
        {
            super.setText(t);
        }
    }
    
    
    /**
     * Resets the undo manager.
     */
    protected void resetUndoManager()
    {
        undo.discardAllEdits();
        undoAction.update();
        redoAction.update();
    }
    
    
    /**
     * Undo Action
     */
    class UndoAction extends AbstractAction
    {
        public UndoAction()
        {
            super("Undo");
            setEnabled(false);
        }
        
        public void actionPerformed(ActionEvent e)
        {
            try
            {
                undo.undo();
            }
            catch (CannotUndoException ex)
            {
                log.warning("Unable to undo: " + ex);
                ex.printStackTrace();
            }
            update();
            redoAction.update();
        }
        
        protected void update()
        {
            if(undo.canUndo())
                setEnabled(true);
            else
                setEnabled(false);
        }
    }
    
    
    /**
     * Redo Action
     */
    class RedoAction extends AbstractAction
    {
        public RedoAction()
        {
            super("Redo");
            setEnabled(false);
        }
        
        public void actionPerformed(ActionEvent e)
        {
            try
            {
                undo.redo();
            }
            catch (CannotRedoException ex)
            {
                log.warning("Unable to redo: " + ex);
                ex.printStackTrace();
            }
            update();
            undoAction.update();
        }
        
        protected void update()
        {
            if(undo.canRedo())
                setEnabled(true);
            else
                setEnabled(false);
        }
    }
    
    
    /**
     * UndoHandler
     */
    class UndoHandler implements UndoableEditListener
    {
        /**
         * Messaged when the Document has created an edit, the edit is
         * added to <code>undo</code>, an instance of UndoManager.
         */
        public void undoableEditHappened(UndoableEditEvent e)
        {
            undo.addEdit(e.getEdit());
            undoAction.update();
            redoAction.update();
        }
    }
    
    
    
    /**
     * Create Actions
     */
    private void createActions()
    {
        undoAction = new UndoAction();
        getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_Z, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()), "Undo");
        getActionMap().put("Undo", undoAction);
        
        redoAction = new RedoAction();
        getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_Y, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()), "Redo");
        getActionMap().put("Redo", redoAction);
    }
    
    
    public JMenuItem undoActionTo(JMenuItem menuItem)
    {
        menuItem.setAction(undoAction);
        menuItem.setMnemonic(KeyEvent.VK_U);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        return menuItem;
    }
    
    
    public JMenuItem redoActionTo(JMenuItem menuItem)
    {
        menuItem.setAction(redoAction);
        menuItem.setMnemonic(KeyEvent.VK_R);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        return menuItem;
    }
    
    
    public JMenuItem cutActionTo(JMenuItem menuItem)
    {
        menuItem.setAction(new LatexEditorKitC.CutAction());
        menuItem.setText("Cut");
        menuItem.setMnemonic(KeyEvent.VK_T);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        return menuItem;
    }
    
    
    public JMenuItem copyActionTo(JMenuItem menuItem)
    {
        menuItem.setAction(new LatexEditorKitC.CopyAction());
        menuItem.setText("Copy");
        menuItem.setMnemonic(KeyEvent.VK_C);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        return menuItem;
    }
    
    
    public JMenuItem pasteActionTo(JMenuItem menuItem)
    {
        menuItem.setAction(new LatexEditorKitC.PasteAction());
        menuItem.setText("Paste");
        menuItem.setMnemonic(KeyEvent.VK_P);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        return menuItem;
    }
    
    
    /**
     * always return true to prevent a horizontal scrollbar
     */
    public boolean getScrollableTracksViewportWidth()
    {
        return true;
    }
    
}
