/*
 * LatexScannerC.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package dk.thrysoee.laeqed.latexeditor;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Simple parser. A MatchC is returned with color, position and length of the token match.
 *
 * @author Jess Thrysoee
 */
public class LatexScannerC
{
    private Matcher currentMatcher;
    private TokenC currentToken;
    
    private final String input;
    private int pos;
    
    private static final List<TokenC> TokenList = new ArrayList<TokenC>();
    
    // TODO should be configurable
    private static final Color BROWN = new Color(128,128,0);
    private static final Color MAGENTA = new Color(192,0,192);
    private static final Color DARKBLUE = new Color(0,0,192);
//    private static final Color DARKGREEN = new Color(0,128,0);
    
    static
    {
        TokenList.add(new TokenC("comment", Pattern.compile("%.*$"), DARKBLUE));
        TokenList.add(new TokenC("command", Pattern.compile("\\\\[a-zA-Z]+"), BROWN));
        TokenList.add(new TokenC("bracket", Pattern.compile("\\{|\\}|\\[|\\]"), MAGENTA));
    }
    
    
    public LatexScannerC(String input)
    {
        this.input = input;
    }
    
    
    public boolean hasNext()
    {
        Matcher m;
        int start = Integer.MAX_VALUE;
        
        for (TokenC t : TokenList)
        {
            m = t.pattern.matcher(input);
            if (m.find(pos))
            {
                if (m.start() < start)   // find first match
                {
                    start = m.start();
                    currentMatcher = m;
                    currentToken = t;
                }
            }
            
        }
        
        if (start != Integer.MAX_VALUE)
        {
            pos = currentMatcher.end();
            return true;
        }
        return false;
    }
    
    
    public MatchC next()
    {
        return new MatchC(currentMatcher, currentToken);
    }
    
    
    public static class MatchC
    {
        public final int start;
        public final int end;
        public final TokenC token;
        
        public MatchC(Matcher m, TokenC token)
        {
            start = m.start();
            end = m.end();
            this.token = token;
        }
    }
    
    
    public static class TokenC
    {
        public final String identifier;
        public final Pattern pattern;
        public final Color color;
        
        public TokenC(String identifier, Pattern pattern, Color color)
        {
            this.identifier = identifier;
            this.pattern = pattern;
            this.color = color;
        }
    }
    
}

