/*
 * LoggingShortFormatterC.java
 *
 * Copyright (C) 2006 Jess Thrysoee
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package dk.thrysoee.utils;

import java.util.logging.Formatter;
import java.util.logging.LogRecord;

/**
 * This is a one line formatter for java.util.logging purposes.
 *
 * Setup logging for the application by adding a system property, e.g.
 * <code>-Djava.util.logging.config.file=logging.properties</code>
 * <p>
 * The logging.properties file could contains something like this,
 *
 * <pre>
 * # java.util.logging config file
 * # Levels are FINEST, FINER, FINE, CONFIG, INFO, WARNING, SEVERE and ALL and OFF
 *
 * handlers = java.util.logging.ConsoleHandler
 * dk.thrysoee.level = FINE
 * java.util.logging.ConsoleHandler.level = FINE
 * java.util.logging.ConsoleHandler.formatter = dk.thrysoee.utils.LoggingShortFormatterC
 * </pre>
 *
 * @author Jess Thrysoee
 */
public class LoggingShortFormatterC extends Formatter
{
    public String format(LogRecord r)
    {
        String className = r.getSourceClassName().substring(r.getSourceClassName().lastIndexOf('.') + 1);
        return String.format("%1$-7s %2$s.%3$s: %4$s%n", r.getLevel(), className, r.getSourceMethodName(), r.getMessage());
    }
}


